
/*
-- SQL Server 2008 and R2 Diagnostic Information Queries
-- Glenn Berry 
-- March 2011
-- http://sqlserverperformance.wordpress.com/
-- Twitter: GlennAlanBerry
*/


/* Calculates average stalls per read, per write, and per total input/output for each database file. 


	Run this twice - sort by read stalls desc, then by write stalls desc.
	
	Microsoft's guidelines want OLTP reads under 20ms, writes under 5ms.
	Real-world guidelines are closer to 100ms reads, 20ms writes.
	If using SSD, should expect to see <1ms both.
*/
SELECT 
	DB_NAME(fs.database_id) AS [Database Name], 
	mf.physical_name, 
	io_stall_read_ms, 
	num_of_reads,
	CAST(io_stall_read_ms/(1.0 + num_of_reads) AS NUMERIC(10,1)) AS [avg_read_stall_ms],io_stall_write_ms, 
	num_of_writes,
	CAST(io_stall_write_ms/(1.0+num_of_writes) AS NUMERIC(10,1)) AS [avg_write_stall_ms],
	[io_stall] AS [total_io_stall_ms], 
	num_of_reads + num_of_writes AS [total_io],
	CAST((io_stall_read_ms + io_stall_write_ms)/(1.0 + num_of_reads + num_of_writes) AS NUMERIC(10,1)) AS [avg_io_stall_ms]
FROM sys.dm_io_virtual_file_stats(null,null) AS fs
	INNER JOIN sys.master_files AS mf 
		ON fs.database_id = mf.database_id
			AND fs.[file_id] = mf.[file_id]
ORDER BY avg_read_stall_ms DESC OPTION (RECOMPILE);

SELECT 
	DB_NAME(fs.database_id) AS [Database Name], 
	mf.physical_name, 
	io_stall_read_ms, 
	num_of_reads,
	CAST(io_stall_read_ms/(1.0 + num_of_reads) AS NUMERIC(10,1)) AS [avg_read_stall_ms],io_stall_write_ms, 
	num_of_writes,
	CAST(io_stall_write_ms/(1.0+num_of_writes) AS NUMERIC(10,1)) AS [avg_write_stall_ms],
	[io_stall] AS [total_io_stall_ms], 
	num_of_reads + num_of_writes AS [total_io],
	CAST((io_stall_read_ms + io_stall_write_ms)/(1.0 + num_of_reads + num_of_writes) AS NUMERIC(10,1)) AS [avg_io_stall_ms]
FROM sys.dm_io_virtual_file_stats(null,null) AS fs
	INNER JOIN sys.master_files AS mf 
		ON fs.database_id = mf.database_id
			AND fs.[file_id] = mf.[file_id]
ORDER BY avg_write_stall_ms DESC OPTION (RECOMPILE);