
/* Procedure cache - http://sqlserverpedia.com/wiki/Queries_in_Procedure_Cache 

Depending on the system's bottleneck, sort by:
 - CPU - total_elapsed_time desc
 - Disk - total_logical_reads desc
 - Blocking - total_logical_writes
*/
SELECT TOP 25 db_name(st.[dbid]) AS [database], total_worker_time/execution_count as AvgCPU, total_worker_time AS TotalCPU
, total_elapsed_time/execution_count as AvgDuration, total_elapsed_time AS TotalDuration  
, total_logical_reads/execution_count as AvgReads, total_logical_reads AS TotalReads
, total_logical_writes/execution_count as AvgWrites, total_logical_writes AS TotalWrites
, execution_count   
, substring(st.text, (qs.statement_start_offset/2)+1  
, ((case qs.statement_end_offset  when -1 then datalength(st.text)  
else qs.statement_end_offset  
end - qs.statement_start_offset)/2) + 1) as txt  
, query_plan
from sys.dm_exec_query_stats as qs  
cross apply sys.dm_exec_sql_text(qs.sql_handle) as st  
cross apply sys.dm_exec_query_plan (qs.plan_handle) as qp 
WHERE db_name(st.[dbid]) = '<database_name,varchar(100),Foo>'
order by total_elapsed_time DESC