USE [IndexBacon];

--=======================================================
--Differences between 'detailed' and 'limited' scan type:
SELECT * 
FROM sys.[dm_db_index_physical_stats](db_id(), NULL, NULL, NULL, 'detailed')
WHERE object_name(object_id) = 'baconbitsbytes';

SELECT *
FROM sys.[dm_db_index_physical_stats](db_id(), NULL, NULL, NULL, 'limited')
WHERE object_name(object_id) = 'baconbitsbytes';


-------- Notice a row is returned for all index levels when using detailed.
/*
	0 for index leaf levels, heaps, and LOB_DATA or ROW_OVERFLOW_DATA allocation units. 
	> 0 for nonleaf index levels. index_level will be the highest at the root level of an index. 
	The nonleaf levels of indexes are only processed when mode = DETAILED. 
*/ 

--=======================================================

--=======================================================
/*
So, what does fragmentation look like?  Let's see a more
user-friendly presentation
*/

SELECT db_name(DDIPS.[database_id]) AS [Database], 
	object_name(DDIPS.[object_id], 
	db_id()) AS [Object],
	I.[name] AS [Index], 
	DDIPS.[alloc_unit_type_desc], 
	DDIPS.[avg_fragmentation_in_percent] AS [Frag %],
	DDIPS.[fragment_count],
	DDIPS.[avg_fragment_size_in_pages],
	DDIPS.[page_count],
	DDIPS.[record_count]
FROM sys.[dm_db_index_physical_stats](db_id(), NULL, NULL, NULL, 'limited') DDIPS
	INNER JOIN sys.[indexes] I ON DDIPS.[object_id] = I.[object_id] AND DDIPS.[index_id] = I.[index_id]
ORDER BY object_name(DDIPS.[object_id]), [DDIPS].[index_id];


--Let's cause some FRAG-MEN-TATION!!!!
USE [IndexBacon];
DBCC SHRINKFILE (N'IndexBacon' , 1);
GO


SELECT db_name(DDIPS.[database_id]) AS [Database], 
	object_name(DDIPS.[object_id], 
	db_id()) AS [Object],
	I.[name] AS [Index], 
	DDIPS.[alloc_unit_type_desc], 
	DDIPS.[avg_fragmentation_in_percent] AS [Frag %],
	DDIPS.[fragment_count],
	DDIPS.[avg_fragment_size_in_pages],
	DDIPS.[page_count],
	DDIPS.[record_count]
FROM sys.[dm_db_index_physical_stats](db_id('IndexBacon'), NULL, NULL, NULL, 'limited') DDIPS
	INNER JOIN sys.[indexes] I ON DDIPS.[object_id] = I.[object_id] AND DDIPS.[index_id] = I.[index_id]
ORDER BY object_name(DDIPS.[object_id]), [DDIPS].[index_id];

--============================================================
/*Cleaning up our toys when done playing with them*/
ALTER DATABASE [IndexBacon] MODIFY FILE ( NAME = N'IndexBacon', SIZE = 100MB )
--============================================================
