--=========================================================
--This is how many days of activity are stored in cache:
--=========================================================
SELECT DATEDIFF(dd,[create_date], GETDATE()) FROM sys.databases WHERE [name] = 'tempdb';


--=========================================================
--Return usage results for all indexes in current database
--=========================================================
SELECT OBJECT_NAME(DDIUS.[object_id]) AS [Object Name], S.[name] AS [index name], S.[rowcnt] AS [index rows], 
	DDIUS.[user_updates], DDIUS.[user_seeks], DDIUS.[user_scans], DDIUS.[user_lookups],
	DDIUS.[system_updates], DDIUS.[system_seeks], DDIUS.[system_scans], DDIUS.[system_lookups]
FROM sys.[dm_db_index_usage_stats] DDIUS 
	INNER JOIN sys.[sysindexes] S ON DDIUS.[object_id] = S.[id] AND DDIUS.[index_id] = S.[indid] 
WHERE DDIUS.[database_id] = db_id()
ORDER BY OBJECT_NAME(DDIUS.[object_id]), S.[name], DDIUS.[user_updates] DESC;
GO


--=========================================================
--Return results for all indexes not being read, but are written to
--=========================================================
SELECT OBJECT_NAME(DDIUS.[object_id]) AS [Object Name], S.[name] AS [index name], S.[rowcnt] AS [index rows], 
	DDIUS.[user_updates], 
	DDIUS.[user_seeks] + DDIUS.[user_scans] + DDIUS.[user_lookups] AS [user_reads]
FROM sys.[dm_db_index_usage_stats] DDIUS 
	INNER JOIN [IndexBacon]..[sysindexes] S ON DDIUS.[object_id] = S.[id] AND DDIUS.[index_id] = S.[indid] 
WHERE DDIUS.[database_id] = db_id() 
	AND DDIUS.[index_id] > 0
	AND DDIUS.[user_seeks] + DDIUS.[user_scans] + DDIUS.[user_lookups] = 0 
	AND DDIUS.[user_updates] > 0 
ORDER BY OBJECT_NAME(DDIUS.[object_id]), S.[name], DDIUS.[user_updates] DESC;
GO


--=========================================================
--Include generation of drop command
--=========================================================
SELECT 
	'['+ DB_NAME() + '].[' + SSU.[name] + '].[' + SO.[name] + ']' AS [statement], 
	SI.name AS [index_name], SI.index_id, SUM(SP.rows) AS [total_rows], 
	(DDIUS.user_seeks + DDIUS.user_scans + DDIUS.user_lookups) AS [user_reads], 
	DDIUS.user_updates AS [user_writes], 
	'DROP INDEX [' + SI.[name] + '] ON [' + SSU.[name] + '].[' + SO.[name] + '] WITH ( ONLINE = OFF )' AS [drop_command]
FROM sys.dm_db_index_usage_stats DDIUS 
	INNER JOIN sys.indexes SI 
		ON DDIUS.OBJECT_ID = SI.OBJECT_ID AND SI.index_id = DDIUS.index_id   
	INNER JOIN sys.partitions SP 
		ON DDIUS.OBJECT_ID = SP.OBJECT_ID AND SP.index_id= DDIUS.index_id 
	INNER JOIN sys.objects SO ON DDIUS.[object_id] = SO.[object_id] 
	INNER JOIN sys.sysusers SSU ON SO.[schema_id] = SSU.UID
WHERE 
	OBJECTPROPERTY(DDIUS.OBJECT_ID,'IsUserTable') = 1 
	AND DDIUS.index_id > 0 
	AND DDIUS.database_id = DB_ID()
GROUP BY SI.name, SI.index_id, 
	user_seeks+ user_scans + user_lookups, user_updates, 
	SSU.[name], SO.[name]
HAVING 
	(DDIUS.user_seeks + DDIUS.user_scans + DDIUS.user_lookups) = 0
ORDER BY SO.[name], SI.[name];
GO


--=========================================================
--Estimate storage impact for unused indexes
--=========================================================
WITH index_space AS
	(
		SELECT object_id, index_id, SUM([record_count]) AS [record_count], 
			SUM([record_count] * [avg_record_size_in_bytes]) [est_index_size_bytes]
		FROM sys.[dm_db_index_physical_stats](db_id(), NULL, NULL, NULL, 'detailed')
		GROUP BY [object_id], [index_id]
	) 
     
SELECT OBJECT_NAME(DDIUS.[object_id]) AS [Object Name], S.[name] AS [index name], 
	S.[rowcnt] AS [leaf-level rows], DDIUS.[user_updates], 
	(DDIUS.[user_seeks] + DDIUS.[user_scans] + DDIUS.[user_lookups]) AS [user_reads],
	I.[record_count], CAST(I.[est_index_size_bytes]	AS bigint) AS [est_index_size_bytes]
FROM sys.[dm_db_index_usage_stats] DDIUS 
	INNER JOIN [IndexBacon]..[sysindexes] S 
		ON DDIUS.[object_id] = S.[id] AND DDIUS.[index_id] = S.[indid] 
	INNER JOIN index_space I 
		ON DDIUS.[object_id] = I.[object_id] AND DDIUS.[index_id] = I.[index_id]
WHERE DDIUS.[database_id] = DB_ID() 
	AND DDIUS.[index_id] > 0
	AND DDIUS.[user_seeks] + DDIUS.[user_scans] + DDIUS.[user_lookups] = 0 
	AND DDIUS.[user_updates] > 0 
ORDER BY OBJECT_NAME(DDIUS.[object_id]), S.[name], DDIUS.[user_updates] DESC;
GO

